# -*- coding: utf-8 -*-

from kinder.core.alloc.helpers import GroupHelper

from gisdo.index import BaseIndex
from gisdo.algorithm import (
    CommonCountAlgorithm, ShortStayAlgorithm,
    CapacitiesHealthNeedAlgorithm, CompensatingAlgorithm,
    HealthGroupAlgorithm)
from gisdo.constants import (
    GOVERNMENT, AGE_CUT_SUBCATEGORY_MAP,
    AGE_CUT_CATEGORY_MAP, AGE_EIGHT_SUBCATEGORY_MAP,
    AGE_EIGHT_CATEGORY_MAP, AGE_CATEGORIES_CUT,
    AGE_CUT_DEFAULT_CATEGORY, ALL_UNIT_TYPES,
    AGE_EIGHT_DEFAULT_CATEGORY, DOU_TYPE_MAP)


class CapacitiesIndex(BaseIndex):
    """
    """

    COMMON_COUNT = 1
    SHORT_STAY = 2
    HEALTH_NEED = 3
    COMPENSATING = 4
    HEALTH = 5

    ALGORITHM_MAP = {
        COMMON_COUNT: CommonCountAlgorithm,
        SHORT_STAY: ShortStayAlgorithm,
        HEALTH_NEED: CapacitiesHealthNeedAlgorithm,
        COMPENSATING: CompensatingAlgorithm,
        HEALTH: HealthGroupAlgorithm
    }

    class Ctx(object):

        kind = 1

        def __init__(self, unit_id):

            self.group_unit = unit_id

    def __init__(self, unit):

        super(CapacitiesIndex, self).__init__(unit, session=None)

        _, self._cache = GroupHelper(
            CapacitiesIndex.Ctx(self.unit.id)).getList()

    @staticmethod
    def _get_age_map(age_category):

        cut_values = AGE_CATEGORIES_CUT.values()

        if age_category in cut_values:
            subcategory_map, category_map, default_category = (
                AGE_CUT_SUBCATEGORY_MAP, AGE_CUT_CATEGORY_MAP,
                AGE_CUT_DEFAULT_CATEGORY)
        else:
            subcategory_map, category_map, default_category = (
                AGE_EIGHT_SUBCATEGORY_MAP, AGE_EIGHT_CATEGORY_MAP,
                AGE_EIGHT_DEFAULT_CATEGORY)

        return subcategory_map, category_map, default_category

    def get_count(
            self, index_type=COMMON_COUNT,
            dou_type=ALL_UNIT_TYPES, **kwargs):

        if dou_type != ALL_UNIT_TYPES:
            if ((not self.dou_type and dou_type != GOVERNMENT) or
                    (self.dou_type and
                     self.dou_type.code not in DOU_TYPE_MAP[dou_type])):
                return 0

        result_list = CapacitiesIndex.get_algorithm(
            index_type)().get_result_set(
                cache=self._cache)

        age_category = kwargs.get('age_category', None)
        subcategory_map, category_map, default_category = self._get_age_map(
            age_category)
        total = 0
        for group in result_list:

            # Если у группы есть возрастная подкатегория,
            # то соотношение проводится по ней
            if group.sub_age_cat:
                if group.sub_age_cat.code in subcategory_map[
                        age_category]:
                    total += group.cntFree
            # в противном случае по возрастной категории,
            # которая обязательна.
            elif group.age_cat:
                if group.age_cat.code in category_map[age_category]:
                    total += group.cntFree
            else:
                if age_category and age_category == default_category:
                    total += group.cntFree
        return total
